@@ -26,9 +26,16 @@
 					  (((guint32)c) <<  8) |  ((guint32)d)      )
 
 #define PACKAGE			"claws-mail"
-#define VERSION			"3.8.1"
-#define PROG_VERSION		"Claws Mail "VERSION
-#define VERSION_NUMERIC		MAKE_NUMERIC_VERSION(3, 8, \
-						     1, 0)
-
+#define PACKAGE_FRIENDLY	"Claws Mail"
+#define VERSION			"3.9.2"
+#define RELEASE_VERSION		"3.9.2"
+#define PROG_VERSION		PACKAGE_FRIENDLY" "VERSION_GIT_FULL
+#ifdef USE_MAINTAINER_MODE
+#define PROG_VERSION_FRIENDLY	PROG_VERSION
+#else
+#define PROG_VERSION_FRIENDLY	PACKAGE_FRIENDLY" "RELEASE_VERSION
+#endif
+#define VERSION_NUMERIC		MAKE_NUMERIC_VERSION(3, 9, \
+						     2, 0)
+#define VERSION_GIT_FULL	"3.9.2-dirty"
 #endif /* __VERSION_H__ */
