@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include <glib.h>
@@ -31,6 +32,7 @@
 #include "menu.h"
 #include "utils.h"
 #include "gtkutils.h"
+#include "defs.h"
 
 #ifdef MAEMO
 #ifdef CHINOOK
@@ -105,6 +107,50 @@ void cm_menu_set_sensitive_full(GtkUIManager *gui_manager, gchar *menu, gboolean
 	g_free(path);
 }
 
+gchar *cm_menu_item_get_shortcut(GtkUIManager *gui_manager, gchar *menu)
+{
+	GtkWidget *widget;
+	gchar *path = g_strdup_printf("/%s/", menu);
+	const gchar *accel = NULL;
+	GtkAccelKey key;
+
+	widget = gtk_ui_manager_get_widget(gui_manager, path);
+	if( !GTK_IS_WIDGET(widget) ) {
+		g_message("Blah, '%s' is not a widget.\n", path);
+	}
+
+	if( !GTK_IS_MENU_ITEM(widget) ) {
+		g_message("Blah, '%s' is not a menu item.\n", path);
+	}
+
+	g_free(path);
+
+	accel = gtk_menu_item_get_accel_path(GTK_MENU_ITEM(widget));
+
+	if (accel && gtk_accel_map_lookup_entry(accel, &key))
+		return gtk_accelerator_get_label (key.accel_key, key.accel_mods);
+	else
+		return g_strdup(_("None"));
+
+}
+
+GtkWidget *cm_menu_item_new_label_from_url(gchar *url)
+{
+	gint len = strlen(url);
+	if (len > MAX_MENU_LABEL_LENGTH) {
+		g_message("Refusing a %d bytes string as menu label\n", len);
+		url[64] = '\0', url[63] = url[62] = url[61] = '.', url[60] = ' ';
+		GtkWidget *newlabel = gtk_menu_item_new_with_label(url);
+		gtk_widget_set_tooltip_markup(GTK_WIDGET(newlabel),
+			_("<span><b>Warning:</b> This URL was too long for displaying and\n"
+			"has been truncated for safety. This message could be\n"
+			"corrupted, malformed or part of some DoS attempt.</span>"));
+		return newlabel;
+	}
+	
+	return gtk_menu_item_new_with_label(url);
+}
+
 void cm_toggle_menu_set_active_full(GtkUIManager *gui_manager, gchar *menu, gboolean active)
 {
 	GtkWidget *widget;
