@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -44,6 +45,10 @@
 #include <X11/SM/SMlib.h>
 #endif
 
+#if HAVE_FLOCK
+#include <sys/file.h>
+#endif
+
 #include "wizard.h"
 #ifdef HAVE_STARTUP_NOTIFICATION
 # define SN_API_NOT_YET_FROZEN
@@ -89,6 +94,9 @@
 #include "account.h"
 #include "procmsg.h"
 #include "inc.h"
+#include "imap.h"
+#include "send_message.h"
+#include "md5.h"
 #include "import.h"
 #include "manage_window.h"
 #include "alertpanel.h"
@@ -100,6 +108,7 @@
 #endif
 #include "compose.h"
 #include "folder.h"
+#include "folder_item_prefs.h"
 #include "setup.h"
 #include "utils.h"
 #include "gtkutils.h"
@@ -115,6 +124,7 @@
 #include "hooks.h"
 #include "menu.h"
 #include "quicksearch.h"
+#include "advsearch.h"
 
 #ifdef HAVE_LIBETPAN
 #include "imap-thread.h"
@@ -197,6 +207,8 @@ typedef enum
 static struct RemoteCmd {
 	gboolean receive;
 	gboolean receive_all;
+	gboolean cancel_receiving;
+	gboolean cancel_sending;
 	gboolean compose;
 	const gchar *compose_mailto;
 	GList *attach_files;
@@ -219,6 +231,7 @@ static struct RemoteCmd {
 	gboolean subscribe;
 	const gchar *subscribe_uri;
 	const gchar *target;
+	gboolean debug;
 } cmd;
 
 SessionStats session_stats;
@@ -607,7 +620,7 @@ static gboolean sc_process_ice_messages (GIOChannel   *source,
 	if (status == IceProcessMessagesIOError) {
 		IcePointer context = IceGetConnectionContext (connection);
 
-		if (context && GTK_IS_OBJECT (context)) {
+		if (context && G_IS_OBJECT(context)) {
 		guint disconnect_id = g_signal_lookup ("disconnect", G_OBJECT_TYPE (context));
 
 		if (disconnect_id > 0)
@@ -761,7 +774,7 @@ static guint win32_log_handler_gtk_id;
 static void win32_print_stdout(const gchar *string)
 {
 	if (win32_debug_fp) {
-		fprintf(win32_debug_fp, string);
+		fprintf(win32_debug_fp, "%s", string);
 		fflush(win32_debug_fp);
 	}
 }
@@ -769,7 +782,7 @@ static void win32_print_stdout(const gchar *string)
 static void win32_print_stderr(const gchar *string)
 {
 	if (win32_debug_fp) {
-		fprintf(win32_debug_fp, string);
+		fprintf(win32_debug_fp, "%s", string);
 		fflush(win32_debug_fp);
 	}
 }
@@ -1068,9 +1081,10 @@ int main(int argc, char *argv[])
 	gint num_folder_class = 0;
 	gboolean asked_for_migration = FALSE;
 	gboolean start_done = TRUE;
-	GtkUIManager *gui_manager = NULL;
 	GSList *plug_list = NULL;
 	gboolean never_ran = FALSE;
+	gboolean mainwin_shown = FALSE;
+
 	START_TIMING("startup");
 
 	sc_starting = TRUE;
@@ -1085,18 +1099,31 @@ int main(int argc, char *argv[])
 		return 0;
 	}
 
-	main_dump_features_list(TRUE);
-
 	prog_version = PROG_VERSION;
 	argv0 = g_strdup(argv[0]);
 
 	parse_cmd_opt(argc, argv);
 
+	sock_init();
+
+	/* check and create unix domain socket for remote operation */
+	lock_socket = prohibit_duplicate_launch();
+	if (lock_socket < 0) {
+#ifdef HAVE_STARTUP_NOTIFICATION
+		if(gtk_init_check(&argc, &argv))
+			startup_notification_complete(TRUE);
+#endif
+		return 0;
+	}
+
+	main_dump_features_list(TRUE);
 	prefs_prepare_cache();
 
 #ifdef CRASH_DIALOG
 	if (cmd.crash) {
+#if !GTK_CHECK_VERSION(3, 0, 0)
 		gtk_set_locale();
+#endif
 		gtk_init(&argc, &argv);
 		crash_main(cmd.crash_params);
 #ifdef G_OS_WIN32
@@ -1110,20 +1137,11 @@ int main(int argc, char *argv[])
 #if (defined linux && defined SIGIO)
 	install_memory_sighandler();
 #endif
-	sock_init();
-
-	/* check and create unix domain socket for remote operation */
-	lock_socket = prohibit_duplicate_launch();
-	if (lock_socket < 0) {
-#ifdef HAVE_STARTUP_NOTIFICATION
-		if(gtk_init_check(&argc, &argv))
-			startup_notification_complete(TRUE);
-#endif
-		return 0;
-	}
 
 	if (cmd.status || cmd.status_full || cmd.search ||
-		cmd.statistics || cmd.reset_statistics) {
+		cmd.statistics || cmd.reset_statistics || 
+		cmd.cancel_receiving || cmd.cancel_sending ||
+		cmd.debug) {
 		puts("0 Claws Mail not running.");
 		lock_socket_remove();
 		return 0;
@@ -1136,7 +1154,9 @@ int main(int argc, char *argv[])
 
 	reset_statistics();
 	
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	gtk_set_locale();
+#endif
 	gtk_init(&argc, &argv);
 
 #ifdef G_OS_WIN32
@@ -1191,11 +1211,13 @@ int main(int argc, char *argv[])
 	hildon_program = HILDON_PROGRAM(hildon_program_get_instance());
 	static_osso_context = osso_context;
 #endif	
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	gtk_widget_set_default_colormap(
 		gdk_screen_get_system_colormap(
 			gdk_screen_get_default()));
+#endif
 
-	gui_manager = gtkut_create_ui_manager();
+	gtkut_create_ui_manager();
 
 	/* Create container for all the menus we will be adding */
 	MENUITEM_ADDUI("/", "Menus", NULL, GTK_UI_MANAGER_MENUBAR);
@@ -1329,11 +1351,21 @@ int main(int argc, char *argv[])
 	prefs_receive_init();
 	prefs_send_init();
 	tags_read_tags();
+	matcher_init();
 #ifdef USE_ENCHANT
 	gtkaspell_checkers_init();
 	prefs_spelling_init();
 #endif
 
+#ifdef G_OS_WIN32
+	if(prefs_common.gtk_theme && strcmp(prefs_common.gtk_theme, DEFAULT_W32_GTK_THEME))
+		gtk_settings_set_string_property(gtk_settings_get_default(),
+			"gtk-theme-name",
+			prefs_common.gtk_theme,
+			"XProperty");
+#endif
+
+
 	sock_set_io_timeout(prefs_common.io_timeout_secs);
 	prefs_actions_read_config();
 	prefs_display_header_read_config();
@@ -1399,13 +1431,6 @@ int main(int argc, char *argv[])
 	gtk_cmclist_freeze(GTK_CMCLIST(mainwin->folderview->ctree));
 	folder_item_update_freeze();
 
-	/* register the callback of unix domain socket input */
-	lock_socket_tag = claws_input_add(lock_socket,
-					G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI,
-					lock_socket_input_cb,
-					mainwin, TRUE);
-
-
 	prefs_account_init();
 	account_read_config_all();
 
@@ -1466,8 +1491,10 @@ int main(int argc, char *argv[])
 
 	/* if crashed, show window early so that the user
 	 * sees what's happening */
-	if (claws_crashed())
+	if (claws_crashed()) {
 		main_window_popup(mainwin);
+		mainwin_shown = TRUE;
+	}
 
 	account_set_missing_folder();
 	folder_set_missing_folders();
@@ -1542,6 +1569,7 @@ int main(int argc, char *argv[])
 		}
 		main_window_cursor_normal(mainwin);
 		main_window_popup(mainwin);
+		mainwin_shown = TRUE;
 		alertpanel_warning(ngettext(
 				     "The following plugin failed to load. "
 				     "Check the Plugins configuration "
@@ -1561,7 +1589,7 @@ int main(int argc, char *argv[])
 	 	plugin_load_standard_plugins ();
 	}
 	/* if not crashed, show window now */
-	if (!claws_crashed()) {
+	if (!mainwin_shown) {
 		/* apart if something told not to show */
 		if (show_at_startup)
 			main_window_popup(mainwin);
@@ -1681,6 +1709,13 @@ int main(int argc, char *argv[])
 		main_window_set_menu_sensitive(mainwin);
 		toolbar_main_set_sensitive(mainwin);
 	}
+
+	/* register the callback of unix domain socket input */
+	lock_socket_tag = claws_input_add(lock_socket,
+					G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI,
+					lock_socket_input_cb,
+					mainwin, TRUE);
+
 	END_TIMING();
 
 	gtk_main();
@@ -1791,6 +1826,7 @@ static void exit_claws(MainWindow *mainwin)
 	
 	plugin_unload_all("GTK2");
 
+	matcher_done();
 	prefs_toolbar_done();
 
 #ifndef USE_NEW_ADDRBOOK
@@ -1918,6 +1954,10 @@ static void parse_cmd_opt(int argc, char *argv[])
 			cmd.receive_all = TRUE;
 		} else if (!strncmp(argv[i], "--receive", 9)) {
 			cmd.receive = TRUE;
+		} else if (!strncmp(argv[i], "--cancel-receiving", 18)) {
+			cmd.cancel_receiving = TRUE;
+		} else if (!strncmp(argv[i], "--cancel-sending", 16)) {
+			cmd.cancel_sending = TRUE;
 		} else if (!strncmp(argv[i], "--compose-from-file", 19)) {
 			const gchar *p = (i+1 < argc)?argv[i+1]:NULL;
 
@@ -2019,6 +2059,8 @@ static void parse_cmd_opt(int argc, char *argv[])
 			cmd.online_mode = ONLINE_MODE_ONLINE;
 		} else if (!strncmp(argv[i], "--offline", 9)) {
 			cmd.online_mode = ONLINE_MODE_OFFLINE;
+		} else if (!strncmp(argv[i], "--toggle-debug", 14)) {
+			cmd.debug = TRUE;
 		} else if (!strncmp(argv[i], "--statistics", 12)) {
 			cmd.statistics = TRUE;
 		} else if (!strncmp(argv[i], "--reset-statistics", 18)) {
@@ -2040,6 +2082,8 @@ static void parse_cmd_opt(int argc, char *argv[])
 					  "                         attached"));
 			g_print("%s\n", _("  --receive              receive new messages"));
 			g_print("%s\n", _("  --receive-all          receive new messages of all accounts"));
+			g_print("%s\n", _("  --cancel-receiving     cancel receiving of messages"));
+			g_print("%s\n", _("  --cancel-sending       cancel sending of messages"));
 			g_print("%s\n", _("  --search folder type request [recursive]\n"
 					  "                         searches mail\n"
 					  "                         folder ex.: \"#mh/Mailbox/inbox\" or \"Mail\"\n"
@@ -2059,6 +2103,7 @@ static void parse_cmd_opt(int argc, char *argv[])
 			g_print("%s\n", _("  --offline              switch to offline mode"));
 			g_print("%s\n", _("  --exit --quit -q       exit Claws Mail"));
 			g_print("%s\n", _("  --debug                debug mode"));
+			g_print("%s\n", _("  --toggle-debug         toggle debug mode"));
 			g_print("%s\n", _("  --help -h              display this help and exit"));
 			g_print("%s\n", _("  --version -v           output version information and exit"));
 			g_print("%s\n", _("  --version-full -V      output version and built-in features information and exit"));
@@ -2249,6 +2294,7 @@ gboolean claws_is_starting(void)
 	return sc_starting;
 }
 
+#ifdef G_OS_UNIX
 /*
  * CLAWS: want this public so crash dialog can delete the
  * lock file too
@@ -2257,23 +2303,42 @@ gchar *claws_get_socket_name(void)
 {
 	static gchar *filename = NULL;
 	const gchar *socket_dir = NULL;
-	
-	if (rc_dir_is_alt())
-		socket_dir = get_rc_dir();
-	else
-		socket_dir = g_get_tmp_dir();
+	gchar md5sum[33];
+
 	if (filename == NULL) {
-		filename = g_strdup_printf("%s%cclaws-mail-%d",
-					   socket_dir, G_DIR_SEPARATOR,
+		struct stat st;
+		socket_dir = g_strdup_printf("%s%cclaws-mail-%d",
+					   g_get_tmp_dir(), G_DIR_SEPARATOR,
 #if HAVE_GETUID
 					   getuid());
 #else
-					   0);						
+					   0);
 #endif
+		if (stat(socket_dir, &st) < 0 && errno != ENOENT) {
+			g_print("Error stat'ing socket_dir %s: %s\n",
+				socket_dir, strerror(errno));
+		} else if (S_ISSOCK(st.st_mode)) {
+			/* old versions used a sock in $TMPDIR/claws-mail-$UID */
+			debug_print("Using legacy socket %s\n", socket_dir);
+			filename = g_strdup(socket_dir);
+			return filename;
+		}
+
+		if (!is_dir_exist(socket_dir) && make_dir(socket_dir) < 0) {
+			g_print("Error creating socket_dir %s: %s\n",
+				socket_dir, strerror(errno));
+		}
+
+		md5_hex_digest(md5sum, get_rc_dir());
+
+		filename = g_strdup_printf("%s%c%s", socket_dir, G_DIR_SEPARATOR,
+					   md5sum);
+		debug_print("Using control socket %s\n", filename);
 	}
 
 	return filename;
 }
+#endif
 
 static gchar *get_crashfile_name(void)
 {
@@ -2295,14 +2360,51 @@ static gint prohibit_duplicate_launch(void)
 	gchar *path;
 
 	path = claws_get_socket_name();
+	/* Try to connect to the control socket */
 	uxsock = fd_connect_unix(path);
 	
 	if (x_display == NULL)
 		x_display = g_strdup(g_getenv("DISPLAY"));
 
 	if (uxsock < 0) {
+#if HAVE_FLOCK
+		gchar *socket_lock;
+		gint lock_fd;
+		gint ret;
+		/* If connect failed, no other process is running.
+		 * Unlink the potentially existing socket, then
+		 * open it. This has to be done locking a temporary
+		 * file to avoid the race condition where another
+		 * process could have created the socket just in
+		 * between.
+		 */
+		socket_lock = g_strconcat(path, ".lock",
+					  NULL);
+		lock_fd = g_open(socket_lock, O_RDWR|O_CREAT, 0);
+		if (lock_fd < 0) {
+			debug_print("Couldn't open %s: %s (%d)\n", socket_lock,
+				strerror(errno), errno);
+			g_free(socket_lock);
+			return -1;
+		}
+		if (flock(lock_fd, LOCK_EX) < 0) {
+			debug_print("Couldn't lock %s: %s (%d)\n", socket_lock,
+				strerror(errno), errno);
+			close(lock_fd);
+			g_free(socket_lock);
+			return -1;
+		}
+#endif
+
 		claws_unlink(path);
-		return fd_open_unix(path);
+		debug_print("Opening socket %s\n", path);
+		ret = fd_open_unix(path);
+#if HAVE_FLOCK
+		flock(lock_fd, LOCK_UN);
+		claws_unlink(socket_lock);
+		g_free(socket_lock);
+#endif
+		return ret;
 	}
 #else
         HANDLE hmutex;
@@ -2331,6 +2433,10 @@ static gint prohibit_duplicate_launch(void)
 		fd_write_all(uxsock, "receive_all\n", 12);
 	} else if (cmd.receive) {
 		fd_write_all(uxsock, "receive\n", 8);
+	} else if (cmd.cancel_receiving) {
+		fd_write_all(uxsock, "cancel_receiving\n", 17);
+	} else if (cmd.cancel_sending) {
+		fd_write_all(uxsock, "cancel_sending\n", 15);
 	} else if (cmd.compose && cmd.attach_files) {
 		gchar *str, *compose_str;
 
@@ -2373,6 +2479,8 @@ static gint prohibit_duplicate_launch(void)
 		fd_write(uxsock, "online\n", 6);
 	} else if (cmd.online_mode == ONLINE_MODE_OFFLINE) {
 		fd_write(uxsock, "offline\n", 7);
+	} else if (cmd.debug) {
+		fd_write(uxsock, "debug\n", 7);
  	} else if (cmd.status || cmd.status_full) {
   		gchar buf[BUFFSIZE];
  		gint i;
@@ -2451,7 +2559,7 @@ static gint prohibit_duplicate_launch(void)
 static gint lock_socket_remove(void)
 {
 #ifdef G_OS_UNIX
-	gchar *filename;
+	gchar *filename, *dirname;
 #endif
 	if (lock_socket < 0) {
 		return -1;
@@ -2464,7 +2572,10 @@ static gint lock_socket_remove(void)
 
 #ifdef G_OS_UNIX
 	filename = claws_get_socket_name();
+	dirname = g_path_get_dirname(filename);
 	claws_unlink(filename);
+	g_rmdir(dirname);
+	g_free(dirname);
 #endif
 
 	return 0;
@@ -2503,8 +2614,6 @@ static void lock_socket_input_cb(gpointer data,
 	MainWindow *mainwin = (MainWindow *)data;
 	gint sock;
 	gchar buf[BUFFSIZE];
-	/* re-use the same quicksearch (& avoid matcher_list mem.leaks) */
-	static QuickSearch *quicksearch = NULL;
 
 	sock = fd_accept(source);
 	fd_gets(sock, buf, sizeof(buf));
@@ -2518,6 +2627,11 @@ static void lock_socket_input_cb(gpointer data,
 				     prefs_common.newmail_notify_manu);
 	} else if (!strncmp(buf, "receive", 7)) {
 		inc_mail(mainwin, prefs_common.newmail_notify_manu);
+	} else if (!strncmp(buf, "cancel_receiving", 16)) {
+		inc_cancel_all();
+		imap_cancel_all();
+	} else if (!strncmp(buf, "cancel_sending", 14)) {
+		send_cancel();
 	} else if (!strncmp(buf, "compose_attach", 14)) {
 		GList *files = NULL, *curr;
 		AttachInfo *ainfo;
@@ -2555,6 +2669,8 @@ static void lock_socket_input_cb(gpointer data,
 		main_window_toggle_work_offline(mainwin, FALSE, FALSE);
 	} else if (!strncmp(buf, "offline", 7)) {
 		main_window_toggle_work_offline(mainwin, TRUE, FALSE);
+	} else if (!strncmp(buf, "debug", 5)) {
+		debug_set_mode(debug_get_mode() ? FALSE : TRUE);
  	} else if (!strncmp(buf, "status-full", 11) ||
  		   !strncmp(buf, "status", 6)) {
  		gchar *status;
@@ -2573,10 +2689,22 @@ static void lock_socket_input_cb(gpointer data,
 		g_snprintf(tmp, sizeof(tmp), _("Session statistics\n"));
  		fd_write_all(sock, tmp, strlen(tmp));
 
-		g_snprintf(tmp, sizeof(tmp), _("Started: %s\n"),
-				ctime(&session_stats.time_started));
+		if (prefs_common.date_format) {
+			struct tm *lt;
+			gint len = 100;
+			gchar date[len];
+
+			lt = localtime(&session_stats.time_started);
+			fast_strftime(date, len, prefs_common.date_format, lt);
+			g_snprintf(tmp, sizeof(tmp), _("Started: %s\n"),
+					lt ? date : ctime(&session_stats.time_started));
+		} else
+			g_snprintf(tmp, sizeof(tmp), _("Started: %s\n"),
+					ctime(&session_stats.time_started));
  		fd_write_all(sock, tmp, strlen(tmp));
 
+ 		fd_write_all(sock, "\n", 1);
+
 		g_snprintf(tmp, sizeof(tmp), _("Incoming traffic\n"));
  		fd_write_all(sock, tmp, strlen(tmp));
 
@@ -2615,37 +2743,44 @@ static void lock_socket_input_cb(gpointer data,
 	} else if (!strncmp(buf, "search ", 7)) {
 		FolderItem* folderItem = NULL;
 		GSList *messages = NULL;
-		gchar *folder_name, *request;
-		QuickSearchType searchType = QUICK_SEARCH_EXTENDED;
+		gchar *folder_name = NULL;
+		gchar *request = NULL;
+		AdvancedSearch *search;
 		gboolean recursive;
-
-		if (quicksearch==NULL)
-			quicksearch = quicksearch_new_nogui();
+		AdvancedSearchType searchType = ADVANCED_SEARCH_EXTENDED;
 		
+		search = advsearch_new();
+
 		folder_name = g_strdup(buf+7);
 		strretchomp(folder_name);
 
-		if (fd_gets(sock, buf, sizeof(buf)) <= 0) {
-			g_free(folder_name);
-			folder_name=NULL;
-		}
-		searchType = quicksearch_type(buf);
-		if (fd_gets(sock, buf, sizeof(buf)) <= 0) {
-			g_free(folder_name);
-			folder_name=NULL;
+		if (fd_gets(sock, buf, sizeof(buf)) <= 0) 
+			goto search_exit;
+
+		switch (toupper(buf[0])) {
+		case 'S': searchType = ADVANCED_SEARCH_SUBJECT; break;
+		case 'F': searchType = ADVANCED_SEARCH_FROM; break;
+		case 'T': searchType = ADVANCED_SEARCH_TO; break;
+		case 'M': searchType = ADVANCED_SEARCH_MIXED; break;
+		case 'G': searchType = ADVANCED_SEARCH_TAG; break;
+		case 'E': searchType = ADVANCED_SEARCH_EXTENDED; break;
 		}
+
+		if (fd_gets(sock, buf, sizeof(buf)) <= 0) 
+			goto search_exit;
+
 		request = g_strdup(buf);
 		strretchomp(request);
 
 		recursive = TRUE;
 		if (fd_gets(sock, buf, sizeof(buf)) > 0)
-			if (buf[0]=='0')
-				recursive = FALSE;
+			recursive = buf[0] != '0';
 
-		debug_print("search: %s %i %s %i\n",folder_name,searchType,request,recursive);
+		debug_print("search: %s %i %s %i\n", folder_name, searchType, request, recursive);
 
 		if (folder_name)
 			folderItem = folder_find_item_from_identifier(folder_name);
+
 		if (folder_name && folderItem == NULL) {
 			debug_print("Unknown folder item : '%s', searching folder\n",folder_name);
 			Folder* folder = folder_find_from_path(folder_name);
@@ -2656,16 +2791,16 @@ static void lock_socket_input_cb(gpointer data,
 		} else {
 			debug_print("%s %s\n",folderItem->name, folderItem->path);
 		}
+
 		if (folderItem != NULL) {
-			quicksearch_set(quicksearch, searchType, request);
-			quicksearch_set_recursive(quicksearch, recursive);
-			search_msgs_in_folders(&messages, quicksearch, folderItem);
+			advsearch_set(search, searchType, request);
+			advsearch_search_msgs_in_folders(search, &messages, folderItem, recursive);
 		} else {
 			g_print("Folder '%s' not found.\n'", folder_name);
 		}
 
 		GSList *cur;
-		for (cur=messages; cur != NULL; cur = cur->next) {
+		for (cur = messages; cur != NULL; cur = cur->next) {
 			MsgInfo* msg = (MsgInfo *)cur->data;
 			gchar *file = procmsg_get_message_file_path(msg);
 			fd_write_all(sock, file, strlen(file));
@@ -2674,10 +2809,12 @@ static void lock_socket_input_cb(gpointer data,
 		}
 		fd_write_all(sock, ".\n", 2);
 
-		if (messages != NULL)
-			procmsg_msg_list_free(messages);
+search_exit:
 		g_free(folder_name);
 		g_free(request);
+		advsearch_free(search);
+		if (messages != NULL)
+			procmsg_msg_list_free(messages);
 	} else if (!strncmp(buf, "exit", 4)) {
 		if (prefs_common.clean_on_exit && !prefs_common.ask_on_clean) {
 			procmsg_empty_all_trash();
