@@ -19,6 +19,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -33,6 +34,7 @@
 #include <time.h>
 
 #include "folder.h"
+#include "folder_item_prefs.h"
 #include "mh.h"
 #include "procmsg.h"
 #include "procheader.h"
@@ -146,6 +148,7 @@ FolderClass *mh_get_class(void)
 		mh_class.type = F_MH;
 		mh_class.idstr = "mh";
 		mh_class.uistr = "MH";
+		mh_class.supports_server_search = FALSE;
 		
 		/* Folder functions */
 		mh_class.new_folder = mh_folder_new;
@@ -173,6 +176,7 @@ FolderClass *mh_get_class(void)
 		mh_class.add_msgs = mh_add_msgs;
 		mh_class.copy_msg = mh_copy_msg;
 		mh_class.copy_msgs = mh_copy_msgs;
+		mh_class.search_msgs = folder_item_search_msgs_local;
 		mh_class.remove_msg = mh_remove_msg;
 		mh_class.remove_msgs = mh_remove_msgs;
 		mh_class.remove_all_msg = mh_remove_all_msg;
@@ -463,7 +467,6 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 	FolderItem *src = NULL;
 	gchar *srcfile;
 	gchar *destfile;
-	gint filemode = 0;
 	FolderItemPrefs *prefs;
 	MsgInfo *msginfo = NULL;
 	MsgInfoList *cur = NULL;
@@ -572,11 +575,6 @@ static gint mh_copy_msgs(Folder *folder, FolderItem *dest, MsgInfoList *msglist,
 		if (prefs && prefs->enable_folder_chmod && prefs->folder_chmod) {
 			if (chmod(destfile, prefs->folder_chmod) < 0)
 				FILE_OP_ERROR(destfile, "chmod");
-
-			/* for mark file */
-			filemode = prefs->folder_chmod;
-			if (filemode & S_IRGRP) filemode |= S_IWGRP;
-			if (filemode & S_IROTH) filemode |= S_IWOTH;
 		}
 		if (relation) {
 			if (g_hash_table_lookup(relation, msginfo) != NULL)
