@@ -23,6 +23,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#include "claws-features.h"
 #endif
 
 #include "defs.h"
@@ -50,6 +51,7 @@
 #include "prefswindow.h"
 #include "prefs_gtk.h"
 #include "plugin.h"
+#include "messageview.h"
 
 enum
 {
@@ -247,10 +249,6 @@ static GHashTable** get_plugin_hash_from_toolbar_type(ToolbarType toolbar_type)
 static void prefs_toolbar_create_widget(PrefsPage *_page, GtkWindow *window, gpointer data)
 {
 	ToolbarPage *prefs_toolbar = (ToolbarPage *) _page;
-	gchar *win_titles[3];
-	win_titles[TOOLBAR_MAIN]    = _("Main toolbar configuration");
-	win_titles[TOOLBAR_COMPOSE] = _("Compose toolbar configuration");  
-	win_titles[TOOLBAR_MSGVIEW] = _("Message view toolbar configuration");  
 
 	prefs_toolbar->window = GTK_WIDGET(window);
 
@@ -370,7 +368,11 @@ static void prefs_toolbar_set_displayed(ToolbarPage *prefs_toolbar)
 
 static void add_item_to_plugin_combo(gpointer key, gpointer data, gpointer combo_box)
 {
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box), (const gchar*)key);
+#else
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_box), (const gchar*)key);
+#endif
 }
 
 static void prefs_toolbar_populate(ToolbarPage *prefs_toolbar)
@@ -389,8 +391,13 @@ static void prefs_toolbar_populate(ToolbarPage *prefs_toolbar)
 			act = (gchar *)cur->data;
 			get_action_name(act, &act_name);
 			
+#if !GTK_CHECK_VERSION(2, 24, 0)
 			gtk_combo_box_append_text(
 				GTK_COMBO_BOX(prefs_toolbar->item_action_combo),
+#else
+			gtk_combo_box_text_append_text(
+				GTK_COMBO_BOX_TEXT(prefs_toolbar->item_action_combo),
+#endif
 				act_name);
 
 			g_free(act_name);
@@ -512,7 +519,11 @@ static void prefs_toolbar_register(GtkButton *button, ToolbarPage *prefs_toolbar
 		}
 				
 		if (item_type == ITEM_FUNCTION) {
+#if !GTK_CHECK_VERSION(2, 24, 0)
 			event = gtk_combo_box_get_active_text(GTK_COMBO_BOX(
+#else
+			event = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(
+#endif
 						prefs_toolbar->item_func_combo));
 						
 			if (is_duplicate(prefs_toolbar, event)) {
@@ -598,7 +609,11 @@ static void prefs_toolbar_substitute(GtkButton *button, ToolbarPage *prefs_toolb
 						  -1);
 		
 		if (item_type == ITEM_FUNCTION) {
+#if !GTK_CHECK_VERSION(2, 24, 0)
 			icon_event = gtk_combo_box_get_active_text(GTK_COMBO_BOX(
+#else
+			icon_event = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(
+#endif
 						prefs_toolbar->item_func_combo));
 						
 			if (is_duplicate(prefs_toolbar, icon_event)
@@ -792,7 +807,11 @@ static void item_type_changed(GtkComboBox *item_type_combo,
 static void action_selection_changed(GtkComboBox *action_combo,
 				ToolbarPage *prefs_toolbar)
 {
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	gchar *text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(
+#else
+	gchar *text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(
+#endif
 			   prefs_toolbar->item_action_combo));
 
 	if(text != NULL) { /* action */
@@ -804,7 +823,11 @@ static void action_selection_changed(GtkComboBox *action_combo,
 static void plugin_selection_changed(GtkComboBox *action_combo,
                 ToolbarPage *prefs_toolbar)
 {
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	gchar *text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(prefs_toolbar->item_plugin_combo));
+#else
+	gchar *text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(prefs_toolbar->item_plugin_combo));
+#endif
 
 	if (text != NULL) { /* action */
 		gtk_entry_set_text(GTK_ENTRY(prefs_toolbar->item_text_entry), text);
@@ -815,7 +838,11 @@ static void plugin_selection_changed(GtkComboBox *action_combo,
 static void func_selection_changed(GtkComboBox *action_combo,
 				ToolbarPage *prefs_toolbar)
 {
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	gchar *text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(
+#else
+	gchar *text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(
+#endif
 			   prefs_toolbar->item_func_combo));
 
 	if(text != NULL) { /* action */
@@ -923,21 +950,33 @@ static void prefs_toolbar_create(ToolbarPage *prefs_toolbar)
 			  (GtkAttachOptions) (GTK_FILL),
 			  (GtkAttachOptions) (0), 0, 0);
 
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	item_action_combo = gtk_combo_box_new_text();
+#else
+	item_action_combo = gtk_combo_box_text_new();
+#endif
 	gtk_widget_set_size_request(item_action_combo, 200, -1);
 	gtk_table_attach (GTK_TABLE (table), item_action_combo, 1, 3, 1, 2,
 			  (GtkAttachOptions) (GTK_FILL),
 			  (GtkAttachOptions) (0), 0, 0);
 			  
 	/* available internal functions */
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	item_func_combo = gtk_combo_box_new_text();
+#else
+	item_func_combo = gtk_combo_box_text_new();
+#endif
 	gtk_widget_set_size_request(item_func_combo, 200, -1);
 	gtk_table_attach (GTK_TABLE (table), item_func_combo, 1, 3, 1, 2,
 			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 			  (GtkAttachOptions) (0), 0, 0);
 	
 	/* plugin-registered items */
+#if !GTK_CHECK_VERSION(2, 24, 0)
 	item_plugin_combo = gtk_combo_box_new_text();
+#else
+	item_plugin_combo = gtk_combo_box_text_new();
+#endif
 	gtk_widget_set_size_request(item_plugin_combo, 200, -1);
 	gtk_table_attach(GTK_TABLE(table), item_plugin_combo, 1, 3, 1, 2,
 			 (GtkAttachOptions) (GTK_FILL),
@@ -1600,11 +1639,9 @@ static gboolean icon_window_button_press(GtkWidget *widget,
 					       GdkEventButton *event,
 					       ToolbarPage *prefs_toolbar )
 {
-	GtkWidget *event_widget, *button;
+	GtkWidget *event_widget;
 	gboolean restore = TRUE;
 
-	button = prefs_toolbar->icon_button;
-
 	/* Test where mouse was clicked */
 	event_widget = gtk_get_event_widget((GdkEvent *)event);
 	if (event_widget != widget) {
